
try:
    # py3
    import tkinter as tk
except:
    # py2
    import Tkinter as tk

from papis.gui.tk.main import PapisWidget
from papis.gui.tk.main import Gui
from papis.gui.tk.prompt import Prompt
from papis.gui.tk.list import PapisList
